@@ -10,6 +10,7 @@ import warnings
 from collections import defaultdict
 from typing import Dict, List, Optional, Set, Tuple
 
+import backoff
 import google.auth
 import google.auth.transport.requests
 import requests
@@ -177,7 +178,7 @@ def load_gke_config(
         token = cred.token
 
     # Return the config data.
-    logit.info(f"Assuming GKE cluster.")
+    logit.info("Assuming GKE cluster.")
     return K8sConfig(
         url=cluster["server"],
         token=token,
@@ -229,10 +230,10 @@ def load_eks_config(
         return (K8sConfig(), True)
 
     # Convert a None value (valid value in YAML) to an empty list of env vars.
-    env_kubeconf = env_kubeconf if env_kubeconf is not None else []
+    env_kubeconf = env_kubeconf if env_kubeconf else []
 
     # Save the certificate to a temporary file. This is only necessary because
-    # the requests library will need a path to the CA file - unfortunately, we
+    # the Requests library will need a path to the CA file - unfortunately, we
     # cannot just pass it the content.
     _, tmp = tempfile.mkstemp(text=False)
     ssl_ca_cert = Filepath(tmp)
@@ -255,9 +256,12 @@ def load_eks_config(
     except (KeyError, yaml.YAMLError):
         logit.error(f"Token manifest from <{cmd}> is corrupt")
         return (K8sConfig(), True)
+    except TypeError:
+        logit.error(f"The YAML token produced by <{cmd}> is corrupt")
+        return (K8sConfig(), True)
 
     # Return the config data.
-    logit.info(f"Assuming EKS cluster.")
+    logit.info("Assuming EKS cluster.")
     return K8sConfig(
         url=cluster["server"],
         token=token,
@@ -298,7 +302,7 @@ def load_minikube_config(fname: Filepath,
         )
 
         # Return the config data.
-        logit.info(f"Assuming Minikube cluster.")
+        logit.info("Assuming Minikube cluster.")
         return K8sConfig(
             url=cluster["server"],
             token="",
@@ -354,7 +358,7 @@ def load_kind_config(fname: Filepath, context: Optional[str]) -> Tuple[K8sConfig
         client_cert = K8sClientCert(crt=p_client_crt, key=p_client_key)
 
         # Return the config data.
-        logit.debug(f"Assuming Minikube/Kind cluster.")
+        logit.debug("Assuming Minikube/Kind cluster.")
         return K8sConfig(
             url=cluster["server"],
             token="",
@@ -546,12 +550,40 @@ def request(
         (dict, int): the JSON response and the HTTP status code.
 
     """
+    # Define the maximum number of tries and exceptions we want to retry on.
+    max_tries = 21
+    web_exceptions = (requests.exceptions.RequestException, )
+
+    def on_backoff(details):
+        """Log a warning whenever we retry."""
+        tries = details["tries"]
+        logit.warning(f"Backing off on {url} (Attempt {tries}/{max_tries-1})")
+
+    """
+    Use linear backoff. The backoff is not exponential because the most
+    prevalent use case for this backoff we have seen so far is to wait for
+    new resource endpoints to become available. These may take a few seconds,
+    or tens of seconds to do so. If we used an exponential strategy we may
+    end up waiting for a very long time for no good reason. The time between
+    backoffs is fairly large to avoid hammering the API. Jitter is disabled
+    because the irregular intervals are irritating in an interactive tool.
+    """
+    @backoff.on_exception(backoff.constant, web_exceptions,
+                          max_tries=max_tries,
+                          interval=3,
+                          max_time=5,
+                          on_backoff=on_backoff,
+                          logger=None,
+                          jitter=None,
+                          )
+    def _call(*args, **kwargs):
+        return client.request(method, url, json=payload, headers=headers, timeout=30)
+
+    # Make the web request via our backoff/retry handler.
     try:
-        ret = client.request(method, url, json=payload, headers=headers, timeout=30)
-    except requests.exceptions.ConnectionError as err:
-        method = err.request.method
-        url = err.request.url
-        logit.error(f"Connection error: {method} {url}")
+        ret = _call()
+    except web_exceptions as err:
+        logit.error(f"{err} ({method} {url})")
         return ({}, True)
 
     try:
