@@ -203,7 +203,7 @@ def parse(
         # Decode the YAML documents in the current file.
         try:
             manifests = list(yaml.safe_load_all(yaml_str))
-        except yaml.scanner.ScannerError as err:
+        except (yaml.parser.ParserError, yaml.scanner.ScannerError) as err:
             logit.error(
                 f"Cannot YAML parse <{fname}>"
                 f" - {err.problem} - Line {err.problem_mark.line}"
@@ -596,7 +596,7 @@ def align_serviceaccount(
         )
 
         """
-        # Do nothing if the ServiceAccount has not "secrets" - should be impossible.
+        # Do nothing if the ServiceAccount has no "secrets" - should be impossible.
         try:
             secrets_dict = manifests[meta]["secrets"]
         except KeyError:
